<?php
$path = getcwd();
$path .= "/includes/";
ini_set("include_path", $path . PATH_SEPARATOR . ini_get("include_path"));

include_once('Auth/Container/DB.php');
class CustomAuthContainer extends Auth_Container_DB {
    /**
     * Constructor of the container class
     *
     * Initate connection to the database via PEAR::DB
     *
     * @param  string Connection data or DB object
     * @return object Returns an error object if something went wrong
     */
    function CustomAuthContainer($dsn)
    {
        $this->_setDefaults();

        if (is_array($dsn)) {
            $this->_parseOptions($dsn);

            if (empty($this->options['dsn'])) {
                PEAR::raiseError('No connection parameters specified!');
            }
        } else {
            $this->options['dsn'] = $dsn;
        }
    }

    /**
     * Get user information from database
     *
     * This function uses the given username to fetch
     * the corresponding login data from the database
     * table. If an account that matches the passed username
     * and password is found, the function returns true.
     * Otherwise it returns false.
     *
     * @param   string Username
     * @param   string Password
     * @param   boolean If true password is secured using an md5 hash
     *                  the frontend and auth are responsible for making sure the container supports
     *                  challenge responce password authenthication
     * @return  mixed  Error object or boolean
     */
    function fetchData($username, $password, $isChallengeResponce=false) {
        //print "Container_DB::fetchData($username, $password, $isChallengeResponce) <br/>\n";
        // Prepare for a database query
        $err = $this->_prepare();
        if ($err !== true) {
            return PEAR::raiseError($err->getMessage(), $err->getCode());
        }

        // Find if db_fields contains a *, if so assume all col are selected
        if (strstr($this->options['db_fields'], '*')) {
            $sql_from = "*";
        }
        else{
            $sql_from = $this->options['usernamecol'] . ", ".$this->options['passwordcol'].$this->options['db_fields'];
        }
        /**
         Old Style, removed to go around the oci8 
         problem 
         See bug 206
         http://pear.php.net/bugs/bug.php?id=206
         
        $query = "SELECT ! FROM ! WHERE ! = ?";
        $query_params = array(
                         $sql_from,
                         $this->options['table'],
                         $this->options['usernamecol'],
                         $username
                         );
        */
        
        $query = "SELECT ".$sql_from.
                " FROM ".$this->options['table'].
                " WHERE ".$this->options['usernamecol']." = '".$this->db->quoteString($username)."'";
        
        $res = $this->db->getRow($query, null, DB_FETCHMODE_ASSOC);
        #print "SQL: $query <br/>\n";
        #print_r($res);

        if (DB::isError($res)) {
            return PEAR::raiseError($res->getMessage(), $res->getCode());
        }
        if (!is_array($res)) {
            $this->activeUser = '';
            return false;
        }
        
	  if($this->options['cryptType'] != 'none') { 
	  	  $password .= '7b949c8716'; 
	  }
        // Perform trimming here before the hashihg
        $password = trim($password, "\r\n");
        $res[$this->options['passwordcol']] = trim($res[$this->options['passwordcol']], "\r\n");
        // If using Challenge Response sha1 the pass with the secret
        if($isChallengeResponce) {
            $res[$this->options['passwordcol']] = sha1($res[$this->options['passwordcol']].$this->_auth_obj->session['loginchallenege']);
            if($this->options['cryptType'] == 'sha1') {
                $res[$this->options['passwordcol']] = sha1($res[$this->options['passwordcol']]);
            }
        }
        if ($this->verifyPassword($password,
                                  $res[$this->options['passwordcol']],
                                  $this->options['cryptType'])) {
            // Store additional field values in the session
            foreach ($res as $key => $value) {
                if ($key == $this->options['passwordcol'] ||
                    $key == $this->options['usernamecol']) {
                    continue;
                }
                // Use reference to the auth object if exists
                // This is because the auth session variable can change so a static call to setAuthData does not make sence
                if (is_object($this->_auth_obj)) {
                    $this->_auth_obj->setAuthData($key, $value);
                } else {
                    Auth::setAuthData($key, $value);
                }
            }
            return true;
        }

        $this->activeUser = $res[$this->options['usernamecol']];
        return false;
    }
}
?>
